/***********************************************************************************************************************
 *
 * BlueBook - use live code examples in your book
 * Copyright (C) 2011-2011 by Tidalwave s.a.s.
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebook.java.net
 * SCM: https://hg.java.net/hg/bluebook~src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebook.filter.java;

import javax.annotation.Nonnull;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import lombok.Getter;
import lombok.Setter;

/***********************************************************************************************************************
 *
 * TODO: should be replaced by a better implementation based on a Java lexical parser
 * 
 * @author  Fabrizio Giudici
 * @version $Id: JavaFilter.java,v 542b4f5de3c7 2011/03/09 12:20:50 fabrizio $
 * @stable
 *
 **********************************************************************************************************************/
public class JavaFilter 
  {
    @Getter @Setter
    private boolean skipHeader;
    
    // FIXME: this actually works only in some simplified cases
    @Getter @Setter
    private boolean skipBlockComments;
    
    @Getter @Setter
    private String includedFragment = "";
    
    @Getter @Setter
    private String excludedFragment = "";
    
    @Getter @Setter
    private boolean skipPackage;
    
    @Getter @Setter
    private boolean skipImports;
    
    public void filter (final @Nonnull Reader r, final @Nonnull Writer w)
      throws IOException
      {
        final BufferedReader br = new BufferedReader(r);
        final PrintWriter pw = new PrintWriter(w);
        boolean inHeader = true;
        boolean inBlockComment = true;
        boolean inIncludedFragment = false;
        boolean inExcludedFragment = false;
        boolean inIgnoredFragment = false;
        boolean prevIsBlank = true;
        
        for (;;)
          {
            final String line = br.readLine();
            
            if (line == null)
              {
                break;  
              }
            
            final String trimmedLine = line.trim();
            boolean skip = false;
            
            if (line.startsWith("package "))
              {
                inHeader = false;
              }
            
            if (trimmedLine.startsWith("/*"))
              {
                inBlockComment = true;  
              }
            
            if (!"".equals(includedFragment) && line.contains("//@bluebook-begin " + includedFragment))
              {
                inIncludedFragment = true;
              }
            
            if (!"".equals(excludedFragment) && line.contains("//@bluebook-begin " + excludedFragment))
              {
                inExcludedFragment = true;
              }
            
            if (line.contains("//@bluebook-ignore-begin"))
              {
                inIgnoredFragment = true;
              }
            
            skip |= skipHeader && inHeader;
            skip |= skipBlockComments && inBlockComment;
            skip |= line.contains("//@bluebook");
            skip |= !"".equals(includedFragment) && !inIncludedFragment;
            skip |= !"".equals(excludedFragment) && inExcludedFragment;
            skip |= inIgnoredFragment;
            skip |= (skipPackage) && line.startsWith("package");
            skip |= (skipImports) && line.startsWith("import ");
            
            final boolean isBlank = trimmedLine.equals("");
            skip |= isBlank && prevIsBlank;
            
            if (!skip)
              {
                pw.println(line);
                prevIsBlank = isBlank;
              }
            
            if (trimmedLine.endsWith("*/"))
              {
                inBlockComment = false;
              } 
            
            if (!"".equals(includedFragment) && line.contains("//@bluebook-end " + includedFragment))
              {
                inIncludedFragment = false;
              }            
            
            if (!"".equals(excludedFragment) && line.contains("//@bluebook-end " + excludedFragment))
              {
                inExcludedFragment = false;
              }                        
            
            if (line.contains("//@bluebook-ignore-end"))
              {
                inIgnoredFragment = false;
              }            
          }
      }
  }
